#if defined(ARDUINO_ARCH_ESP8266)
#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>
typedef ESP8266WebServer WiFiWebServer;
#elif defined(ARDUINO_ARCH_ESP32)
#include <WiFi.h>
#include <WebServer.h>
typedef WebServer WiFiWebServer;
#endif
#include <LSM303AGR_ACC_Sensor.h>
#include <LSM303AGR_MAG_Sensor.h>

#if defined(ARDUINO_SAM_DUE)
#define DEV_I2C Wire1   //Define which I2C bus is used. Wire1 for the Arduino Due
#define SerialPort Serial
#else
#define DEV_I2C Wire    //Or Wire
#define SerialPort Serial
#endif

// Components.
LSM303AGR_ACC_Sensor Acc(&DEV_I2C);
LSM303AGR_MAG_Sensor Mag(&DEV_I2C);

WiFiWebServer server;

float magnetometerX, magnetometerY, magnetometerZ;
float accelerometerX, accelerometerY, accelerometerZ;
float temperature;

const char *ssid = "JioFiber-4G";
const char *password = "1234567890";

void updateSensorData() {
  int32_t accelerometer[3];
  Acc.GetAxes(accelerometer);

  // Read temperature LSM303AGR.
  float temp;
  Acc.GetTemperature(&temp);

  // Read magnetometer LSM303AGR.
  int32_t magnetometer[3];
  Mag.GetAxes(magnetometer);

  magnetometerX = magnetometer[0];
  magnetometerY = magnetometer[1];
  magnetometerZ = magnetometer[2];

  accelerometerX = accelerometer[0];
  accelerometerY = accelerometer[1];
  accelerometerZ = accelerometer[2];

  temperature = temp;
}

void setup() {
  delay(1000);
  Serial.begin(115200);
  Serial.println();

  // Connect to Wi-Fi
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(250);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());

  // Initialize I2C bus.
  DEV_I2C.begin();

  // Initialize components.
  Acc.begin();
  Acc.Enable();
  Acc.EnableTemperatureSensor();
  Mag.begin();
  Mag.Enable();

  // Responder of root page and apply page handled directly from WebServer class.
  server.on("/", HTTP_GET, []() {
    updateSensorData();  // Update sensor data in real time

    String content = R"(
<!DOCTYPE html>
<html>
<head>
  <meta charset="UTF-8" name="viewport" content="width=device-width, initial-scale=1">
  <style>
    body {
      font-family: Arial, sans-serif;
      margin: 20px;
    }
    .sensor-box {
      border: 2px solid #ddd;
      padding: 20px;
      margin: 10px;
      text-align: center;
      font-size: 24px;
      background-color: #f8f8f8;  /* Light cream color */
    }
    .magnetometer {
      color: #3366cc;  /* Dark blue color */
    }
    .accelerometer {
      color: #cc3366;  /* Dark pink color */
    }
    .temperature {
      color: #33cc33;  /* Dark green color */
    }
  </style>
  <script>
    function updateValues() {
      fetch("/sensor-values")
        .then(response => response.text())
        .then(data => {
          const [magnetometerX, magnetometerY, magnetometerZ, accelerometerX, accelerometerY, accelerometerZ, temperature] = data.split(",");
          document.getElementById("magnetometerX").innerText = magnetometerX;
          document.getElementById("magnetometerY").innerText = magnetometerY;
          document.getElementById("magnetometerZ").innerText = magnetometerZ;
          document.getElementById("accelerometerX").innerText = accelerometerX;
          document.getElementById("accelerometerY").innerText = accelerometerY;
          document.getElementById("accelerometerZ").innerText = accelerometerZ;
          document.getElementById("temperature").innerText = temperature;
        });
    }

    setInterval(updateValues, 1000); // Update every 1000 milliseconds (1 second)
  </script>
</head>
<body>
<div class="sensor-box magnetometer">
  <strong>Magnetometer</strong><br>
  X: <span id="magnetometerX">0.0</span><br>
  Y: <span id="magnetometerY">0.0</span><br>
  Z: <span id="magnetometerZ">0.0</span><br>
</div>
<div class="sensor-box accelerometer">
  <strong>Accelerometer</strong><br>
  X: <span id="accelerometerX">0.0</span><br>
  Y: <span id="accelerometerY">0.0</span><br>
  Z: <span id="accelerometerZ">0.0</span><br>
</div>
<div class="sensor-box temperature">
  <strong>Temperature</strong><br>
  <span id="temperature">0.0</span> &#8451;<br>
</div>
</body>
</html>
    )";
    server.send(200, "text/html", content);
  });

  server.on("/sensor-values", HTTP_GET, []() {
    updateSensorData();
    String sensorValues = String(magnetometerX) + "," + String(magnetometerY) + "," + String(magnetometerZ) + "," +
                          String(accelerometerX) + "," + String(accelerometerY) + "," + String(accelerometerZ) + "," +
                          String(temperature);
    server.send(200, "text/plain", sensorValues);
  });

  server.begin();
}

void loop() {
  server.handleClient();
}
